/*
An example of how to do CS 321 HW4 problem 4
using pup I/O.

Orion Sky Lawlor, olawlor@acm.org, 2006/04/07 (Public Domain)
*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include <map>
#include <string>
#include "lib/inc.h"

#include <fstream>


/********** tiny little example pup library (modify as needed!) ******/
/*
PUP: Pack / UnPack
	A pup_er accepts your object's data fields.
It's allowed to read *or* write to your fields.
*/
class pup_er {
public:
	virtual void bytes(void *data,int nbytes) =0;
};

// pup routines for builtin types.
//   For portability, these could use network byte order.
void pup(pup_er &p,int &c) {
	p.bytes((char *)&c,sizeof(int));
}
void pup(pup_er &p,double &c) {
	p.bytes((char *)&c,sizeof(double));
}

// pup routines for STL types.
void pup(pup_er &p,std::string &c) {
	int len=c.size();
	pup(p,len);
	c.resize(len);
	p.bytes(&c[0],len*sizeof(char));
}
template <class T>
void pup(pup_er &p,std::vector<T> &v) {
	int len=v.size();
	pup(p,len);
	v.resize(len);
	for (int i=0;i<len;i++)
		pup(p,v[i]);
	
}

/* A pup_er that pups objects into a disk file (writing pup_er) */
class pup_to_disk : public pup_er {
public:
	std::ostream &s;
	pup_to_disk(std::ostream &s_) :s(s_) {}
	void bytes(void *data,int nbytes){
		s.write((char *)data,nbytes);
	}
};
/* A pup_er that pups objects out of a disk file (reading pup_er) */
class pup_from_disk : public pup_er {
public:
	std::istream &s;
	pup_from_disk(std::istream &s_) :s(s_) {}
	void bytes(void *data,int nbytes){
		s.read((char *)data,nbytes);
	}
};

/*************** end of pup library, beginning example code *********/




struct dude {
public:
	std::string name;
	std::string address;
	double karma;
	std::vector<std::string> blogs;
};
void pup(pup_er &p,dude &d) {
	pup(p,d.name);
	pup(p,d.address);
	pup(p,d.karma);
	pup(p,d.blogs); /* amazingly, this works! */
}


/* Make up a random string for testing */
std::string makeString(int length) {
	std::string s="";
	for (int i=0;i<length;i++) {
		if (length<20) s+='A'+(rand()%24);
		else if ((rand()&0xff)<5) s+='\n';
		else s+=32+(rand()%(127-32));
	}
	return s;
}

/* Make up some random dude objects for testing */
std::vector<dude> makeDudes(void) {
	std::vector<dude> dudes;
	int i,n=2+(rand()&3);
	for (i=0;i<n;i++) {
		dude f;
		f.name=makeString(6);
		f.address=makeString(150);
		f.karma=rand()&511;
		for (int j=0;j<(rand()&3);j++)
			f.blogs.push_back(makeString(18));
		dudes.push_back(f);
	}
	return dudes;
}
void printDude(dude &f) {
	std::cout<<"Dude:\n";
	std::cout<<"  name='"<<f.name<<"'\n";
	std::cout<<"  address='"<<f.address<<"'\n";
	std::cout<<"  kharma="<<f.karma<<"\n";
	for (unsigned int i=0;i<f.blogs.size();i++) 
		std::cout<<"  blog='"<<f.blogs[i]<<"'\n";
}
void printDudes(std::vector<dude> &dudes) {

	for (unsigned int i=0;i<dudes.size();i++) printDude(dudes[i]);
}

/* Print out this int in binary.
  FIXME: use big-endian output for portability... */
void writeInt(std::ofstream &s,int &v) {
	s.write((char *)&v,sizeof(v));
}

/* Print out this string in binary */
void writeString(std::ofstream &s,std::string &str) {
	int len=str.size();
	writeInt(s,len);
	s.write(&str[0],len);
}

/* Write and read all these dude objects to/from this disk file.
   You MUST write both these routines, and CANNOT change any other routine,
   (in particular, you must not change foo!)
*/
void writeDudes(const char *toFile,std::vector<dude> dudes) {
	std::ofstream s(toFile);
	pup_to_disk p(s);
	pup(p,dudes);
}

void readDudes(const char *fromFile,std::vector<dude> &dudes) {
	std::ifstream s(fromFile);
	pup_from_disk p(s);
	pup(p,dudes);
}

int main() {
	srand(3);

/* Make up some dudes, and write them to disk */
	writeDudes("dudes.bin",makeDudes());

/* Read the dudes we just wrote */
	std::vector<dude> dudes;
	readDudes("dudes.bin",dudes);
	if (dudes.size()==0) std::cout<<"Woa--I didn't read any dudes?\n";
	printDudes(dudes);
	return 0;
}




